<?php

namespace App\Helpers;

use App\User;
use App\HomePages;
use App\BranchContacts;
use App\FasAgendaPresentasi;
use App\FasEvents;
use App\FasFranchisee;
use App\YoutubeVideo;
use Carbon\Carbon;

class LocalHandler
{
    function __construct()
    {
    }


    private $handled_methods = [
        'logout',
        'listBranchContacts',
        'listHomePages',
        'getEvents',
        'getAgendaPresentasi',
        'getExpiringContracts',
        'getVirtualEvents',
        'getYoutubeVideos'
    ];

    public function has_handled_method (string $method) {
        return in_array($method, $this->handled_methods);
    }

    const JSONSUCCESS = ["status" => 'T', "message" => "Success"];

    public function handle_post(string $method, array $params, User $user)
    {
        switch ($method) {
            case 'logout':
                $user->tokens()->delete();  //delete session
                return self::JSONSUCCESS;

            case 'listHomePages':
                return $this->handle_list_home_pages();
            case 'listBranchContacts':
                return $this->handle_list_branch_contacts();
            case 'getVirtualEvents':
                return $this->getVirtualEvents($user);
            case 'getExpiringContracts':
                return $this->getExpiringContracts($user);
            case 'getYoutubeVideos':
                return $this->get_youtube_videos();                    
        }
    }

    private function handle_list_home_pages(){
        $homepages = HomePages::all();
        return [
            'status' => 'T',
            'message' => 'Ok',
            'payload' => ['list' => $homepages]
        ];
    }

    private function handle_list_branch_contacts(){
        $contacts = BranchContacts::all();
        return [
            'status' => 'T',
            'message' => 'Ok',
            'payload' => ['list' => $contacts]
        ];
    }

   
    private function getVirtualEvents(User $user) {
        $dateToday = Carbon::today();
        $events = FasEvents::whereDate('tgl_akhir', '>=', $dateToday)
            ->where('id_jenis', 5)
            ->get(['id_event', 'id_jenis', 'nama_event', 'tgl_mulai', 'tgl_akhir',
                'lokasi', 'deskripsi', 'organizer']);

        $agenda = FasAgendaPresentasi::whereDate('tanggal', '>=', $dateToday)
            ->with('fasKaryawan:nik,nama_lengkap')
            ->get(['id_agenda', 'tanggal', 'jam', 'deskripsi', 'nik_karyawan']);

        return [
            'status' => 'T',
            'message' => 'Ok',
            'payload' => [
                'listEvents' => $events, 
                'listAgenda' => $agenda
            ]
        ];
    }
       
    private function getExpiringContracts(User $user) {
        $email = $user->email;
        $franchisee = FasFranchisee::where('email', $email)->first();
        if ($franchisee == null) {
            return [
                'status' => 'F',
                'message' => "Franchisee tidak ditemukan",
                'displayMessage' => "Franchisee $email tidak ditemukan di FAS",
                'payload' => []
            ];
        }
        $listToko = $franchisee->fasToko()->get();
        $listPerjanjian = [];
        $listMasaSewa = [];
        foreach ($listToko as $toko) {
            $kodeToko = $toko->kode_toko;
            if (isset($kodeToko)){
                $perjanjians = $toko->fasPerjanjians()
                ->with(['fasToko:kode_toko,nama_toko'])
                ->get(['id_perjanjian',
                    'kode_toko', 'jenis_perjanjian', 'no_reg',
                    'pks1', 'tgl_awal_pks1', 'tgl_akhir_pks1',
                    'pks2', 'tgl_awal_pks2', 'tgl_akhir_pks2',
                    'pks3', 'tgl_awal_pks3', 'tgl_akhir_pks3',
                ]);

                foreach($perjanjians as $perj) {
                    $listPerjanjian[] = $perj;
                }

                $progressChecklists = $toko->fasChecklists()->get(['id_ms', 
                    'kode_toko', 'nama_toko',
                    'tgl_sewa_awal', 'tgl_sewa_akhir', 'nilai_sewa']);
                foreach($progressChecklists as $ms) {
                $listMasaSewa[] = $ms;
                }
            } 
        }
        return [
            'status' => 'T',
            'message' => 'Ok',
            'payload' => [
                'listPerjanjian' => $listPerjanjian,
                'listMasaSewa' => $listMasaSewa,
            ]
        ];
    }

    private function get_youtube_videos() {
        return ['status' => 'T', 'message' => 'OK', 'payload'=> ['list' => YoutubeVideo::all()]];
    }

}