<?php


namespace App\Helpers;


use JsonSerializable;

class ResponseFormat implements JsonSerializable
{
    /** @var string */
    private $status;
    /** @var string */
    private $message;
    /** @var string */
    private $displayMessage;
    /** @var mixed */
    private $payload;
    //private $token;

    /** @var string */
    private $username;

    /**
     * ResponseFormat constructor.
     * @param string $status
     * @param string $message
     * @param string $displayMessage
     */
    public function __construct(string $status, string $message, string $displayMessage)
    {
        $this->status = $status;
        $this->message = $message;
        $this->displayMessage = $displayMessage;
        $this->payload = [];
    }

    /**
     * @return string
     */
    public function getStatus(): string
    {
        return $this->status;
    }

    /**
     * @param string $status
     */
    public function setStatus(string $status): void
    {
        $this->status = $status;
    }

    /**
     * @return string
     */
    public function getMessage(): string
    {
        return $this->message;
    }

    /**
     * @param string $message
     */
    public function setMessage(string $message): void
    {
        $this->message = $message;
    }

    /**
     * @return string
     */
    public function getDisplayMessage(): string
    {
        return $this->displayMessage;
    }

    /**
     * @param string $displayMessage
     */
    public function setDisplayMessage(string $displayMessage): void
    {
        $this->displayMessage = $displayMessage;
    }

    /**
     * @return array
     */
    public function getPayload()
    {
        return $this->payload;
    }

    /**
     * @param array $payload
     */
    public function setPayload($payload): void
    {
        $this->payload = $payload;
    }

    public function isSuccess() : bool
    {
        return in_array($this->status, ['T', 'V', 'W']);
    }

    /**
     * @return string
     */
    public function getUsername(): string
    {
        return $this->username;
    }

    /**
     * @param string $username
     */
    public function setUsername(string $username): void
    {
        $this->username = $username;
    }


    public function jsonSerialize()
    {
        $json = array_merge($this->payload, [
            'status' => $this->status,
            'message' => $this->message,
            'displayMessage' => $this->displayMessage,
        ]);
        if (isset($this->username)) {
            $json['User'] = $this->username;
        };
        return $json;
    }


    public static function success()
    {
        return new ResponseFormat('T', 'OK', 'Success');
    }

    public static function expired()
    {
        return new ResponseFormat('EXP', 'Expired', 'GATEWAY: Expired Security Token');
    }

    public static function forbidden() {
        return new ResponseFormat('F', 'Forbidden', 'Forbidden function call');
    }

    public static function notLoggedIn() {
        return new ResponseFormat('L', 'Forbidden', 'Please Log In');
    }

    public static function notImpl(string $fn='') {
        return new ResponseFormat('N', 'NotImplemented', 'Fungsi belum tersedia: '.$fn);
    }

}
