<?php

namespace App\Helpers;

use App\User;
use Carbon\Carbon;
use Illuminate\Support\Facades\Hash;
use Symfony\Component\HttpFoundation\ParameterBag;

class SessionControl
{
    /** @var Requester  */
    private $requester;
    /** @var int */
    private $expire_minutes;

    function __construct(Requester $requester)
    {
        $this->requester = $requester;
        $this->expire_minutes = (env('EXPIRE_MINUTES')>0) ? intval(env('EXPIRE_MINUTES')) : 30;
    }

    public function doLogin($email, $password) : ResponseFormat {

        $response = $this->requester->login($email, $password);
        $payload = $response->getPayload();
        if ($response->isSuccess()){
            // register new session
            $user = User::where('email', '=', $email)->first();
            if ($user === null) {
                $user = new User();
                $user->email = $email;
                $user->name ='Franchisee';
            }
            $user->password = Hash::make($password);
            $user->apiAccessToken = $payload['accessToken'];
            $user->apiSessionId = $payload['sessionId'];
            $user->apiLastUpdate = date('Y-m-d H:i:s');
            $user->last_refreshed = date('Y-m-d H:i:s');
            $user->save();
            $token = $user->createToken('mobileApp')->plainTextToken;
            $payload['token'] = $token;
        }
        unset($payload['accessToken']);
        unset($payload['sessionId']);
        $response->setPayload($payload);

        return $response;

    }

    public function doPost(ParameterBag $params, User $user) {
        if ($this->checkTokenExpired($user)) {
            $user->delete();
            return ResponseFormat::expired();
        }

        $method = $params->get('method');
        $paramArray = [];
        foreach ($params->all() as $key => $value) {
            $paramArray[$key] = $value;
        }
        $response = $this->requester->form($method, $paramArray, $user);
        return $response;

    }

    private function checkTokenExpired(User $user) : bool {
        if ($user->last_refreshed !== null) {
            $currentDateTime = Carbon::now();
            $userLastRefresh = Carbon::createFromFormat('Y-m-d H:i:s', $user->last_refreshed);
            $expire = $userLastRefresh->addMinutes( $this->expire_minutes );
            if ($currentDateTime->lte($expire)) {
                $user->last_refreshed = $currentDateTime;
                $user->save();
                return false;
            }
        }
        return true;
    }

    public function doPostPublic(ParameterBag $params) {
        $method = $params->get('method');
        $paramArray = [];
        foreach ($params->all() as $key => $value) {
            $paramArray[$key] = $value;
        }
        $response = $this->requester->formPublic($method, $paramArray);
        return $response;
    }


}

